aboutsummaryrefslogtreecommitdiff
diff options
context:
space:
mode:
-rw-r--r--src/pl/plpgsql/src/expected/plpgsql_simple.out11
-rw-r--r--src/pl/plpgsql/src/pl_exec.c12
-rw-r--r--src/pl/plpgsql/src/sql/plpgsql_simple.sql12
3 files changed, 30 insertions, 5 deletions
diff --git a/src/pl/plpgsql/src/expected/plpgsql_simple.out b/src/pl/plpgsql/src/expected/plpgsql_simple.out
index 7b22e60f198..da351873e74 100644
--- a/src/pl/plpgsql/src/expected/plpgsql_simple.out
+++ b/src/pl/plpgsql/src/expected/plpgsql_simple.out
@@ -118,3 +118,14 @@ select simplecaller();
44
(1 row)
+-- Check handling of simple expression in a scrollable cursor (bug #18859)
+do $$
+declare
+ p_CurData refcursor;
+ val int;
+begin
+ open p_CurData scroll for select 42;
+ fetch p_CurData into val;
+ raise notice 'val = %', val;
+end; $$;
+NOTICE: val = 42
diff --git a/src/pl/plpgsql/src/sql/plpgsql_simple.sql b/src/pl/plpgsql/src/sql/plpgsql_simple.sql
index 143bf09dce4..72d8afe4500 100644
--- a/src/pl/plpgsql/src/sql/plpgsql_simple.sql
+++ b/src/pl/plpgsql/src/sql/plpgsql_simple.sql
@@ -102,3 +102,15 @@ as $$select 22 + 22$$;
select simplecaller();
select simplecaller();
+
+-- Check handling of simple expression in a scrollable cursor (bug #18859)
+
+do $$
+declare
+ p_CurData refcursor;
+ val int;
+begin
+ open p_CurData scroll for select 42;
+ fetch p_CurData into val;
+ raise notice 'val = %', val;
+end; $$;
diff --git a/src/pl/plpgsql/src/pl_exec.c b/src/pl/plpgsql/src/pl_exec.c
index d4377ceecbf..e8205275036 100644
--- a/src/pl/plpgsql/src/pl_exec.c
+++ b/src/pl/plpgsql/src/pl_exec.c
@@ -8302,10 +8302,12 @@ exec_save_simple_expr(PLpgSQL_expr *expr, CachedPlan *cplan)
/*
* Ordinarily, the plan node should be a simple Result. However, if
* debug_parallel_query is on, the planner might've stuck a Gather node
* atop that. The simplest way to deal with this is to look through the
* Gather node. The Gather node's tlist would normally contain a Var
* referencing the child node's output, but it could also be a Param, or
* it could be a Const that setrefs.c copied as-is.
*/
plan = stmt->planTree;
for (;;)
@@ -8323,7 +8325,7 @@ exec_save_simple_expr(PLpgSQL_expr *expr, CachedPlan *cplan)
((Result *) plan)->resconstantqual == NULL);
break;
}
- else if (IsA(plan, Gather))
+ else if (IsA(plan, Gather) || IsA(plan, Material))
{
Assert(plan->lefttree != NULL &&
plan->righttree == NULL &&
@@ -8302,10 +8302,12 @@ exec_save_simple_expr(PLpgSQL_expr *expr, CachedPlan *cplan)
* Ordinarily, the plan node should be a simple Result. However, if
* debug_parallel_query is on, the planner might've stuck a Gather node
- * atop that. The simplest way to deal with this is to look through the
- * Gather node. The Gather node's tlist would normally contain a Var
- * referencing the child node's output, but it could also be a Param, or
- * it could be a Const that setrefs.c copied as-is.
+ * atop that; and/or if this plan is for a scrollable cursor, the planner
+ * might've stuck a Material node atop it. The simplest way to deal with
+ * this is to look through the Gather and/or Material nodes. The upper
+ * node's tlist would normally contain a Var referencing the child node's
+ * output, but it could also be a Param, or it could be a Const that
+ * setrefs.c copied as-is.
*/
plan = stmt->planTree;